# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

import unittest

from requre.record_and_replace import make_generic


class ApplyCommonCase(unittest.TestCase):
    def test_simple_use(self):
        @make_generic
        def add_one(fce):
            def fce_to_return():
                return fce() + 1

            return fce_to_return

        @add_one
        def simple_use():
            return 0

        assert simple_use() == 1

    def test_parenthesis_use(self):
        @make_generic
        def add_one(fce):
            def fce_to_return():
                return fce() + 1

            return fce_to_return

        @add_one()
        def parenthesis_use():
            return 0

        assert parenthesis_use() == 1

    def test_parenthesis_use_with_args(self):
        @make_generic
        def add_something(add):
            def decorator_to_return(fce):
                def fce_to_return():
                    return fce() + add

                return fce_to_return

            return decorator_to_return

        @add_something(1)
        def parenthesis_use_with_args():
            return 0

        assert parenthesis_use_with_args() == 1

    def test_parenthesis_use_with_kwargs(self):
        @make_generic
        def add_something(add):
            def decorator_to_return(fce):
                def fce_to_return():
                    return fce() + add

                return fce_to_return

            return decorator_to_return

        @add_something(add=1)
        def parenthesis_use_with_kwargs():
            return 0

        assert parenthesis_use_with_kwargs() == 1

    def test_class_simple_use(self):
        @make_generic
        def add_one(fce):
            def fce_to_return(*args, **kwargs):
                return fce(*args, **kwargs) + 1

            return fce_to_return

        @add_one
        class SimpleUse:
            def test_simple_use(self):
                # We match `test.*` methods by default
                return 0

        assert SimpleUse().test_simple_use() == 1

    def test_class_parenthesis(self):
        @make_generic
        def add_one(fce):
            def fce_to_return(*args, **kwargs):
                return fce(*args, **kwargs) + 1

            return fce_to_return

        @add_one()
        class SimpleUse:
            def test_simple_use(self):
                # We match `test.*` methods by default
                return 0

        assert SimpleUse().test_simple_use() == 1

    def test_class_parenthesis_use_with_args(self):
        @make_generic
        def add_something(add):
            def decorator_to_return(fce):
                def fce_to_return(*args, **kwargs):
                    return fce(*args, **kwargs) + add

                return fce_to_return

            return decorator_to_return

        @add_something(1)
        class SimpleUse:
            def test_simple_use(self):
                # We match `test.*` methods by default
                return 0

        assert SimpleUse().test_simple_use() == 1

    def test_class_parenthesis_use_with_kwargs(self):
        @make_generic
        def add_something(add):
            def decorator_to_return(fce):
                def fce_to_return(*args, **kwargs):
                    return fce(*args, **kwargs) + add

                return fce_to_return

            return decorator_to_return

        @add_something(add=1)
        class SimpleUse:
            def test_simple_use(self):
                return 0

        assert SimpleUse().test_simple_use() == 1

    def test_class_parenthesis_use_with_regexp_method_pattern_not_set(self):
        @make_generic
        def add_something(add):
            def decorator_to_return(fce):
                def fce_to_return(*args, **kwargs):
                    return fce(*args, **kwargs) + add

                return fce_to_return

            return decorator_to_return

        @add_something(add=1)
        class SimpleUse:
            def simple_use(self):
                return 0

            def test_simple_use(self):
                # We match `test.*` methods by default
                return 0

        assert SimpleUse().simple_use() == 0
        assert SimpleUse().test_simple_use() == 1

    def test_class_parenthesis_use_with_regexp_method_pattern_set(self):
        @make_generic
        def add_something(add):
            def decorator_to_return(fce):
                def fce_to_return(*args, **kwargs):
                    return fce(*args, **kwargs) + add

                return fce_to_return

            return decorator_to_return

        @add_something(add=1, regexp_method_pattern="simple.*")
        class SimpleUse:
            def simple_use(self):
                return 0

            def test_simple_use(self):
                # We match `test.*` methods by default
                return 0

        assert SimpleUse().simple_use() == 1
        assert SimpleUse().test_simple_use() == 0

    def test_double_use_on_decorator(self):
        @make_generic
        @make_generic
        def add_one(fce):
            def fce_to_return():
                return fce() + 1

            return fce_to_return

        @add_one
        def simple_use():
            return 0

        assert simple_use() == 1

    def test_chain_of_decorators(self):
        @make_generic
        def plus_one(fce):
            def fce_to_return():
                return fce() + 1

            return fce_to_return

        @make_generic
        def add_one(fce):
            return plus_one(fce)

        @add_one
        def simple_use():
            return 0

        assert simple_use() == 1

    def test_parenthesis_double_use_with_args(self):
        @make_generic
        @make_generic
        def add_something(add):
            def decorator_to_return(fce):
                def fce_to_return():
                    return fce() + add

                return fce_to_return

            return decorator_to_return

        @add_something(1)
        def parenthesis_use_with_args():
            return 0

        assert parenthesis_use_with_args() == 1

    def test_use_with_args_chain_decorators(self):
        @make_generic
        def add_something(add):
            def decorator_to_return(fce):
                def fce_to_return():
                    return fce() + add

                return fce_to_return

            return decorator_to_return

        @make_generic
        def add_one(fce):
            return add_something(1)(fce)

        @add_one
        def parenthesis_use_with_args():
            return 0

        assert parenthesis_use_with_args() == 1

    def test_parenthesis_use_with_args_not_set(self):
        """
        When using with optional arguments,
        you need to use `if _func is None:....` trick in your decorator.
        """

        @make_generic
        def add_something(_func=None, add=1):
            def decorator_to_return(fce):
                def fce_to_return():
                    return fce() + add

                return fce_to_return

            if _func is None:
                # @add_something()
                return decorator_to_return
            else:
                # @add_something
                return decorator_to_return(_func)

        @add_something()
        def parenthesis_use_with_args():
            return 0

        assert parenthesis_use_with_args() == 1

    def test_simple_use_with_args_not_set(self):
        """
        When using with optional arguments,
        you need to use `if _func is None:....` trick in your decorator.
        """

        @make_generic
        def add_something(_func=None, add=1):
            def decorator_to_return(fce):
                def fce_to_return():
                    return fce() + add

                return fce_to_return

            if _func is None:
                # @add_something()
                return decorator_to_return
            else:
                # @add_something
                return decorator_to_return(_func)

        @add_something
        def parenthesis_use_with_args():
            return 0

        assert parenthesis_use_with_args() == 1



# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

import requests
import unittest
from requre.online_replacing import record_requests_for_all_methods


@record_requests_for_all_methods()
class BaseTest(unittest.TestCase):
    def cassette_setup(self, cassette):
        pass
        # do what ever you want with cassette setup

    def test(self):
        response = requests.get("http://example.com")
        self.assertIn("This domain is for use", response.text)



# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

import logging
import os
from unittest import TestCase
from requre.storage import PersistentObjectStorage
from requre.cassette import StorageKeysInspectOuter, StorageKeysInspectSimple
from requre.utils import get_datafile_filename

logger = logging.getLogger(__name__)


class RequreTestCase(TestCase):
    """
    This is base unittest TestCase what will help you to store requre storage files
    It will create directory structre test_data/TEST_FILE_NAME in a directory where
    test is stored. inside this subdir. there is created storage file via calling
    self.id()
    suffixed by yaml
    """

    def setUp(self):
        super().setUp()
        response_file = get_datafile_filename(self)
        os.makedirs(os.path.dirname(response_file), mode=0o777, exist_ok=True)
        self.cassette = PersistentObjectStorage().cassette
        self.cassette.storage_file = response_file

    def tearDown(self):
        self.cassette.dump()
        super().tearDown()


class RequreTestCaseOuterKeys(RequreTestCase):
    """
    This is base unittest TestCase what will help you to store requre storage files
    It will create directory structre test_data/TEST_FILE_NAME in a directory where
    test is stored. inside this subdir. there is created storage file via calling
    self.id()
    suffixed by yaml

    Using Simplier Storage keys
    It tries to avoid storing of unwanted stack information in your test, testing stack or project.
    It uses PWD to differentiate what is wanted and unwanted.
    """

    def setUp(self):
        super().setUp()
        self.cassette.data_miner.key_stategy_cls = StorageKeysInspectOuter


class RequreTestCaseSimple(RequreTestCase):
    """
    This is base unittest TestCase what will help you to store requre storage files
    It will create directory structre test_data/TEST_FILE_NAME in a directory where
    test is stored. inside this subdir. there is created storage file via calling
    self.id()
    suffixed by yaml

    it uses very simple keys for storage files, should be more robust from perspective
    of running test, less robust from perspective of test debugging.
    Beause it can match antother requrest if there will be more queries.
    """

    def setUp(self):
        super().setUp()
        self.cassette.data_miner.key_stategy_cls = StorageKeysInspectSimple



# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

import functools
import logging
import os
from typing import Any, Optional
from warnings import warn

from requre.cassette import Cassette, CassetteExecution
from requre.objects import ObjectStorage
from requre.record_and_replace import (
    make_generic,
    replace_module_match_with_multiple_decorators,
)
from requre.simple_object import Simple

logger = logging.getLogger(__name__)


class MkTemp(Simple):
    """
    decorate mktemp method wrapper
    """

    pass


class MkDTemp(Simple):
    """
    decorate mkdtemp method wrapper
    """

    def from_serializable(self, data: Any) -> Any:
        os.makedirs(data, exist_ok=True)
        return data


class TempFile(ObjectStorage):
    """
    replace system tempfile module with own predictable names implementation
     of temp files for mocking

     Warn: Replaced by new implementations MkDTemp and MkTemp classes
    """

    root = "/tmp"
    prefix = "static_tmp"
    _cassette: Cassette = None
    _cassette_file = None
    counter = 0
    dir_suffix = "file_storage"

    @classmethod
    def set_cassette(cls, value: Cassette):
        if value.storage_file != cls._cassette_file:
            # cassette file may be same, but storage file could change,
            # be avare of that in case of counter
            cls._cassette_file = value.storage_file
            cls.counter = 0
        cls._cassette = value

    @classmethod
    def next(cls):
        cls.counter += 1
        return cls.counter

    @classmethod
    def _get_name(cls, prefix: Optional[str] = None) -> str:
        if not cls._cassette:
            cls._cassette = cls.get_cassette()
        cls.set_cassette(cls._cassette)
        filename = os.path.join(
            cls.root,
            os.path.basename(cls.get_cassette().storage_file),
            f"{prefix or cls.prefix}_{cls.next()}",
        )
        os.makedirs(os.path.dirname(filename), mode=0o777, exist_ok=True)
        logger.debug(f"Use name for tempfile: ${filename}")
        return filename

    @classmethod
    def mktemp(cls, cassette: Optional[Cassette] = None) -> Any:
        """
        Decorator what will store return value of function/method as file and will store content

        :param cassette: Cassette instance to pass inside object to work with
        :return: CassetteExecution class with function and cassette instance

        """
        warn("Please replace it by MkTemp.decorator_plain()")
        casex = CassetteExecution()
        casex.cassette = cassette or cls.get_cassette()
        casex.obj_cls = cls

        def internal(func):
            @functools.wraps(func)
            def __mktemp(*args, **kwargs):
                output = cls._mktemp(*args, **kwargs)
                return output

            return __mktemp

        casex.function = internal
        return casex

    @classmethod
    def mkdtemp(cls, cassette: Optional[Cassette] = None) -> Any:
        """
        Decorator what will store return value of function/method as file and will store content

        :param cassette: Cassette instance to pass inside object to work with
        :return: CassetteExecution class with function and cassette instance

        """
        warn("Please replace it by class mkdtemp MkDTemp.decorator_plain()")
        casex = CassetteExecution()
        casex.cassette = cassette or cls.get_cassette()
        casex.obj_cls = cls

        def internal(func):
            @functools.wraps(func)
            def __mkdtemp(*args, **kwargs):
                output = cls._mkdtemp(*args, **kwargs)
                return output

            return __mkdtemp

        casex.function = internal
        return casex

    @classmethod
    def _mktemp(cls, prefix: Optional[str] = None) -> str:
        return cls._get_name(prefix)

    @classmethod
    def _mkdtemp(cls, prefix: Optional[str] = None) -> str:
        name = cls._get_name(prefix)
        os.makedirs(name)
        return name


@make_generic
def record_tempfile_module(
    _func=None,
    cassette: Optional[Cassette] = None,
):
    """Records tempfile.mkdtemp and tempfile.mktemp calls."""
    decorators = [
        ("tempfile.mkdtemp", MkDTemp.decorator_plain()),
        ("tempfile.mktemp", MkTemp.decorator_plain()),
    ]
    record_tempfile_decorator = replace_module_match_with_multiple_decorators(
        *decorators,
        cassette=cassette,
    )

    if _func is not None:
        return record_tempfile_decorator(_func)
    else:
        return record_tempfile_decorator



# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

import contextlib
from importlib.metadata import PackageNotFoundError, distribution

from requre.base_testclass import RequreTestCase
from requre.record_and_replace import record, replace
from requre.cassette import Cassette
from requre.simple_object import Simple, Tuple
from requre.guess_object import Guess
from requre.helpers.files import StoreFiles
from requre.import_system import UpgradeImportSystem

with contextlib.suppress(PackageNotFoundError):
    __version__ = distribution(__name__).version

__all__ = [
    record.__name__,
    replace.__name__,
    RequreTestCase.__name__,
    Cassette.__name__,
    Simple.__name__,
    Tuple.__name__,
    Guess.__name__,
    StoreFiles.__name__,
    UpgradeImportSystem.__name__,
]



# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

import tempfile
from requre.cassette import Cassette

keys = ["basic", "keys", 0.1]
value1 = {"cat": "tom", "mouse": "jerry"}
value2 = {"cat1": "pa", "cat2": "pi"}

cassette = Cassette()
cassette.storage_file = tempfile.mktemp()

cassette.store(keys, value1, metadata={})
cassette.store(keys, value2, metadata={})
cassette.dump()

cassette.storage_object = {}
cassette.load()
print(cassette.storage_object)
print(cassette.read(keys))
print(cassette.read(keys))



# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

import os
import shutil
import tempfile
import unittest

import git

from requre.helpers import (
    record_git_module,
    record_tempfile_module,
)
from requre.online_replacing import record_requests


@record_git_module
@record_tempfile_module
@record_requests
class ApplyCommonCase(unittest.TestCase):
    def setUp(self) -> None:
        self._tempdir = None
        self.git_url = "git@github.com:packit/hello-world.git"

    @property
    def tempdir(self):
        if not self._tempdir:
            self._tempdir = tempfile.mkdtemp()
        return self._tempdir

    def tearDown(self) -> None:
        shutil.rmtree(self.tempdir)

    def test_git(self):
        """When regeneration, tempdir will change, so change the expected output in last assert"""
        repo = git.Repo.clone_from(self.git_url, to_path=self.tempdir)
        repo.remotes[0].pull()
        repo.remotes[0].fetch()
        print(">>", repo.remotes[0].pull, repo.remotes[0].pull.__module__)
        repo.remotes[0].push()
        print(self.tempdir)
        self.assertEqual("/tmp/tmpv1xm25f2", self.tempdir)
        self.assertIn("hello.spec", os.listdir(self.tempdir))



# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

import random
import unittest
from requre import record


@record(what="random.random")
class Test(unittest.TestCase):
    def test(self):
        self.assertEqual(0.8682724996217076, random.random())



# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT


import datetime
import json
from contextlib import contextmanager
from typing import Any, Dict, List, Optional

import httpx

from requre.cassette import Cassette
from requre.objects import ObjectStorage
from requre.record_and_replace import make_generic, recording, replace


class HTTPXRequestResponseHandling(ObjectStorage):
    __response_keys = ["status_code", "encoding"]
    __ignored = ["cookies"]
    __response_keys_special = ["next_request", "headers", "_elapsed", "_content"]
    __store_indicator = "__store_indicator"
    __implicit_encoding = "utf-8"

    def __init__(
        self,
        store_keys: list,
        cassette: Optional[Cassette] = None,
        response_headers_to_drop=None,
    ) -> None:
        # replace request if given as key and use prettier url
        for index, key in enumerate(store_keys):
            if isinstance(key, httpx.Request):
                store_keys[index] = str(key.url)
                store_keys.insert(index, key.method)
        super().__init__(store_keys, cassette=cassette)
        self.response_headers_to_drop = response_headers_to_drop or []

    def write(
        self, response: httpx.Response, metadata: Optional[Dict] = None
    ) -> httpx.Response:
        super().write(response, metadata)
        # TODO: disabled for now, improve next handling if we find it makes sense
        # if getattr(response, "next"):
        #    self.write(getattr(response, "next"))
        return response

    def read(self):
        data = super().read()
        # TODO: disabled for now, improve next handling if we find it makes sense
        # if getattr(data, "next"):
        #    data._next = self.read()
        return data

    def to_serializable(self, response: httpx.Response) -> Any:
        output = dict()
        for key in self.__response_keys:
            output[key] = getattr(response, key)
        for key in self.__response_keys_special:
            if key == "headers":
                headers_dict = dict(response.headers)
                for header in self.response_headers_to_drop:
                    if header in headers_dict:
                        headers_dict[header] = None
                output[key] = headers_dict
            if key == "_elapsed":
                output[key] = response.elapsed.total_seconds()
            if key == "_content":
                what_store = response._content  # type: ignore
                encoding = response.encoding or self.__implicit_encoding
                try:
                    what_store = what_store.decode(encoding)  # type: ignore
                    try:
                        what_store = json.loads(what_store)
                        indicator = 2
                    except json.decoder.JSONDecodeError:
                        indicator = 1
                except (ValueError, AttributeError):
                    indicator = 0
                output[key] = what_store
                output[self.__store_indicator] = indicator
            if key == "next_request":
                output[key] = None
                if getattr(response, "next_request") is not None:
                    output[key] = self.store_keys
        return output

    def from_serializable(self, data: Any) -> httpx.Response:
        # Process the content
        encoding = data["encoding"] or self.__implicit_encoding

        indicator = data[self.__store_indicator]
        content, text, deserialized_json = None, None, None
        if indicator == 0:
            content = data["_content"]  # raw data
        elif indicator == 1:
            text = data["_content"]  # encoded text
        elif indicator == 2:
            deserialized_json = data["_content"]  # JSON
        else:
            raise TypeError("Invalid type of encoded content.")

        response = httpx.Response(
            status_code=data["status_code"],
            headers=data["headers"],
            content=content,
            text=text,
            json=deserialized_json,
        )
        response.encoding = encoding
        response.elapsed = datetime.timedelta(seconds=data.get("elapsed", 0))
        response.next_request = data.get("next_request")

        return response

    @classmethod
    def decorator_all_keys(
        cls,
        storage_object_kwargs=None,
        cassette: Cassette = None,
        response_headers_to_drop=None,
    ) -> Any:
        """
        Class method for what should be used as decorator of import replacing system
        This use all arguments of function as keys

        :param func: Callable object
        :param storage_object_kwargs: forwarded to the storage object
        :param response_headers_to_drop: list of header names we don't want to save with response
                                            (Will be replaced to `None`.)
        :param cassette: Cassette instance to pass inside object to work with
        :return: CassetteExecution class with function and cassette instance
        """
        storage_object_kwargs = storage_object_kwargs or {}
        if response_headers_to_drop:
            storage_object_kwargs["response_headers_to_drop"] = response_headers_to_drop
        return super().decorator_all_keys(
            storage_object_kwargs,
            cassette=cassette,
        )

    @classmethod
    def decorator(
        cls,
        *,
        item_list: list,
        map_function_to_item=None,
        storage_object_kwargs=None,
        cassette: Cassette = None,
        response_headers_to_drop=None,
    ) -> Any:
        """
        Class method for what should be used as decorator of import replacing system
        This use list of selection of *args or **kwargs as arguments of function as keys

        :param item_list: list of values of *args nums,  **kwargs names to use as keys
        :param map_function_to_item: dict of function to apply to keys before storing
                                  (have to be listed in item_list)
        :param storage_object_kwargs: forwarded to the storage object
        :param response_headers_to_drop: list of header names we don't want to save with response
                                        (Will be replaced to `None`.)
        :param cassette: Cassette instance to pass inside object to work with
        :return: CassetteExecution class with function and cassette instance
        """
        storage_object_kwargs = storage_object_kwargs or {}
        if response_headers_to_drop:
            storage_object_kwargs["response_headers_to_drop"] = response_headers_to_drop
        return super().decorator(
            item_list=item_list,
            map_function_to_item=map_function_to_item,
            storage_object_kwargs=storage_object_kwargs,
            cassette=cassette,
        )

    @classmethod
    def decorator_plain(
        cls,
        storage_object_kwargs=None,
        cassette: Cassette = None,
        response_headers_to_drop=None,
    ) -> Any:
        """
        Class method for what should be used as decorator of import replacing system
        This use no arguments of function as keys

        :param func: Callable object
        :param storage_object_kwargs: forwarded to the storage object
        :param response_headers_to_drop: list of header names we don't want to save with response
                                          (Will be replaced to `None`.)
        :param cassette: Cassette instance to pass inside object to work with
        :return: CassetteExecution class with function and cassette instance
        """
        storage_object_kwargs = storage_object_kwargs or {}
        if response_headers_to_drop:
            storage_object_kwargs["response_headers_to_drop"] = response_headers_to_drop
        return super().decorator_plain(
            storage_object_kwargs=storage_object_kwargs,
            cassette=cassette,
        )


@make_generic
def record_httpx(
    _func=None,
    response_headers_to_drop: Optional[List[str]] = None,
    cassette: Optional[Cassette] = None,
):
    """
    Decorator which can be used to store all httpx requests to a file
    and replay responses on the next run.

    - The matching is based on `url`.
    - Removes tokens from the url when saving if needed.

    Can be used with or without parenthesis.

    :param _func: can be used to decorate function (with, or without parenthesis).
    :param response_headers_to_drop: list of header names we don't want to save with response
                                        (Will be replaced to `None`.)
    :param storage_file: str - storage file to be passed to cassette instance if given,
                               else it creates new instance
    :param cassette: Cassette instance to pass inside object to work with
    """

    response_headers_to_drop = response_headers_to_drop or []
    replace_decorator = replace(
        what="httpx._client.Client.send",
        cassette=cassette,
        decorate=HTTPXRequestResponseHandling.decorator(
            item_list=[1],
            response_headers_to_drop=response_headers_to_drop,
            cassette=cassette,
        ),
    )

    if _func is not None:
        return replace_decorator(_func)
    else:
        return replace_decorator


@contextmanager
def recording_httpx(
    response_headers_to_drop: Optional[List[str]] = None, storage_file=None
):
    """
    Context manager which can be used to store all httpx requests to a file
    and replay responses on the next run.

    - The matching is based on `url`.
    - Removes tokens from the url when saving if needed.

    :param _func: can be used to decorate function (with, or without parenthesis).
    :param response_headers_to_drop: list of header names we don't want to save with response
                                        (Will be replaced to `None`.)
    :param storage_file: file for reading and writing data in storage_object
    """
    with recording(
        what="httpx._client.Client.send",
        decorate=HTTPXRequestResponseHandling.decorator(
            item_list=[1],
            response_headers_to_drop=response_headers_to_drop,
        ),
        storage_file=storage_file,
    ) as cassette:
        yield cassette



# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

import pytest

from requre.storage import PersistentObjectStorage
from tests.testbase import network_connection_available

pytest_plugins = ["requre.pytest_fixtures"]


@pytest.mark.skipif(not network_connection_available(), reason="No network connection")
def test_record_requests_fixture(record_requests_fixture):
    import requests

    # ensure that storage_file is  not propagated to Singleton
    assert (
        PersistentObjectStorage().cassette.storage_file
        != record_requests_fixture.storage_file
    )
    assert (
        record_requests_fixture.storage_file.name == "test_record_requests_fixture.yaml"
    )
    requests.get("https://google.com")


@pytest.mark.skipif(not network_connection_available(), reason="No network connection")
def test_record_requests_fixture_different_call(record_requests_fixture):
    import requests

    assert (
        record_requests_fixture.storage_file.name
        == "test_record_requests_fixture_different_call.yaml"
    )
    requests.get("https://fedoraproject.org")


@pytest.mark.skipif(not network_connection_available(), reason="No network connection")
def test_record_requests_fixture_write(
    remove_storage_file, record_requests_fixture, remove_storage_file_after
):
    import requests

    assert remove_storage_file == record_requests_fixture.storage_file
    assert remove_storage_file == remove_storage_file_after
    assert (
        record_requests_fixture.storage_file.name
        == "test_record_requests_fixture_write.yaml"
    )
    requests.get("https://google.com")


@pytest.mark.skipif(not network_connection_available(), reason="No network connection")
def test_record_requests_fixture_different_call_write(
    remove_storage_file, record_requests_fixture, remove_storage_file_after
):
    import requests

    assert remove_storage_file == record_requests_fixture.storage_file
    assert remove_storage_file == remove_storage_file_after
    assert (
        record_requests_fixture.storage_file.name
        == "test_record_requests_fixture_different_call_write.yaml"
    )
    requests.get("https://fedoraproject.org")



# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

import importlib
import unittest

from requre.exceptions import ItemNotInStorage
from requre.helpers.httpx_response import HTTPXRequestResponseHandling
from requre.utils import StorageMode
from tests.testbase import BaseClass, network_connection_available


class StoreAnyRequest(BaseClass):
    domain = "https://example.com/"

    def setUp(self) -> None:
        super().setUp()
        self.httpx = importlib.import_module("httpx")
        self.post_orig = getattr(self.httpx, "post")

    def tearDown(self) -> None:
        super().tearDown()
        setattr(self.httpx, "post", self.post_orig)

    @unittest.skipIf(not network_connection_available(), "No network connection")
    def testRawCall(self):
        """
        Test if is class is able to explicitly write and read request handling
        """
        # example.com domain is not accepting POST requests anymore
        # using stg.packit.dev instead
        keys = [
            "https://stg.packit.dev/api/webhooks/github",
        ]
        sess = HTTPXRequestResponseHandling(store_keys=keys)
        response = self.httpx.post(*keys)
        sess.write(response)

        response_after = sess.read()
        self.assertIsInstance(response_after, self.httpx.Response)
        message = (
            "Did not attempt to load JSON data because the request "
            "Content-Type was not 'application/json'."
        )
        self.assertNotIn(message, str(sess.get_cassette().storage_object))
        self.assertIn(message, response_after.text)

    @unittest.skipIf(not network_connection_available(), "No network connection")
    def testExecuteWrapper(self):
        """
        test if it is able to use explicit decorator_all_keys for storing request handling
        :return:
        """
        response_before = HTTPXRequestResponseHandling.execute_all_keys(
            self.httpx.post, self.domain, cassette=self.cassette
        )
        self.cassette.dump()
        self.cassette.mode = StorageMode.read
        response_after = HTTPXRequestResponseHandling.execute_all_keys(
            self.httpx.post, self.domain, cassette=self.cassette
        )
        self.assertEqual(response_before.text, response_after.text)
        self.assertRaises(
            Exception,
            HTTPXRequestResponseHandling.execute_all_keys,
            self.httpx.post,
            self.domain,
            cassette=self.cassette,
        )

    @unittest.skipIf(not network_connection_available(), "No network connection")
    def testFunctionDecorator(self):
        """
        Test main purpose of the class, decorate post function and use it then
        """
        self.httpx.post = HTTPXRequestResponseHandling.decorator_all_keys()(
            self.httpx.post
        )
        response_before = self.httpx.post(self.domain)
        self.cassette.dump()
        self.cassette.mode = StorageMode.read

        response_after = self.httpx.post(self.domain)
        self.assertEqual(response_before.text, response_after.text)
        self.assertRaises(Exception, self.httpx.post, self.domain)

    @unittest.skipIf(not network_connection_available(), "No network connection")
    def testFunctionDecoratorNotFound(self):
        """
        Check if it fails with Exception in case request is not stored
        """
        self.httpx.post = HTTPXRequestResponseHandling.decorator_all_keys()(
            self.httpx.post
        )
        self.cassette.dump()
        self.cassette.mode = StorageMode.read
        self.assertRaises(Exception, self.httpx.post, self.domain, data={"a": "b"})

    @unittest.skipIf(not network_connection_available(), "No network connection")
    def testFunctionCustomFields(self):
        """
        Test if it is able to use partial storing of args, kwargs
        prepare to avoid leak authentication to data
        """
        self.httpx.post = HTTPXRequestResponseHandling.decorator(item_list=[0])(
            self.httpx.post
        )
        response_before = self.httpx.post(self.domain)
        response_google_before = self.httpx.post(
            "http://www.google.com", data={"a": "b"}
        )
        self.cassette.dump()
        self.cassette.mode = StorageMode.read

        response_after = self.httpx.post(self.domain)
        response_google_after = self.httpx.post("http://www.google.com")
        self.assertEqual(response_before.text, response_after.text)
        self.assertEqual(response_google_before.text, response_google_after.text)
        self.assertRaises(Exception, self.httpx.post, self.domain)

    @unittest.skipIf(not network_connection_available(), "No network connection")
    def testFunctionCustomFieldsWrong(self):
        """
        Check exceptions if using partial keys storing
        """
        self.httpx.post = HTTPXRequestResponseHandling.decorator(
            item_list=["url", "data"]
        )(self.httpx.post)
        self.httpx.post(self.domain, data={"a": "b"})
        response_2 = self.httpx.post(self.domain, data={"c": "d"})
        self.cassette.dump()
        self.cassette.mode = StorageMode.read

        self.assertRaises(Exception, self.httpx.post, self.domain, data={"x": "y"})
        self.assertRaises(ItemNotInStorage, self.httpx.post, self.domain)
        response_2_after = self.httpx.post(self.domain, data={"c": "d"})
        self.assertEqual(response_2.text, response_2_after.text)

    @unittest.skipIf(not network_connection_available(), "No network connection")
    def testFunctionCustomFieldsCheckKeys(self):
        self.httpx.post = HTTPXRequestResponseHandling.decorator(
            item_list=["url"], map_function_to_item={"url": lambda x: x[0:10]}
        )(self.httpx.post)
        self.httpx.post(self.domain)
        self.httpx.post("http://www.google.com", data={"a": "b"})
        self.cassette.dump()
        self.cassette.mode = StorageMode.read
        print(">>>", self.cassette.storage_object)
        self.assertIn(
            "https://ex",
            self.cassette.storage_object["unittest.case"]["tests.test_httpx"][
                "requre.objects"
            ]["requre.cassette"]["httpx"]["post"],
        )
        self.assertIn(
            "http://www",
            self.cassette.storage_object["unittest.case"]["tests.test_httpx"][
                "requre.objects"
            ]["requre.cassette"]["httpx"]["post"],
        )

    @unittest.skipIf(not network_connection_available(), "No network connection")
    def testFunctionFilterHeaders(self):
        self.httpx.post = HTTPXRequestResponseHandling.decorator(
            item_list=["url"],
            response_headers_to_drop=["date"],
        )(self.httpx.post)
        self.httpx.post(self.domain)
        self.httpx.post("http://www.google.com", data={"a": "b"})
        self.cassette.dump()
        self.cassette.mode = StorageMode.read

        saved_item = self.cassette.storage_object["unittest.case"]["tests.test_httpx"][
            "requre.objects"
        ]["requre.cassette"]["httpx"]["post"]["http://www.google.com"][0]

        self.assertIn("headers", saved_item["output"])
        self.assertIsNone(saved_item["output"]["headers"]["date"])

    @unittest.skipIf(not network_connection_available(), "No network connection")
    def testFunctionFilterUnknownHeaders(self):
        self.httpx.post = HTTPXRequestResponseHandling.decorator(
            item_list=["url"],
            response_headers_to_drop=["NotKnownHeader"],
        )(self.httpx.post)
        self.httpx.post(self.domain)
        self.httpx.post("http://www.google.com", data={"a": "b"})
        self.cassette.dump()
        self.cassette.mode = StorageMode.read

        saved_item = self.cassette.storage_object["unittest.case"]["tests.test_httpx"][
            "requre.objects"
        ]["requre.cassette"]["httpx"]["post"]["http://www.google.com"][0]

        self.assertIn("headers", saved_item["output"])
        self.assertNotIn("NotKnownHeader", saved_item["output"]["headers"])



# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

from typing import Any
from git.repo.base import Repo as RepoOrigin
from requre.objects import ObjectStorage


class Repo(ObjectStorage):
    store_items = [
        "git_dir",
        "working_dir",
        "_working_tree_dir",
        "_common_dir",
        "_bare",
    ]

    def to_serializable(self, obj: RepoOrigin) -> Any:
        output = dict()
        for item in self.store_items:
            output[item] = getattr(obj, item)
        return output

    def from_serializable(self, data: Any) -> Any:
        out = RepoOrigin(data[self.store_items[0]])
        for item in self.store_items:
            setattr(out, item, data[item])
        return out



# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

import builtins
import functools
from enum import Enum
from typing import Any, Callable, List, Optional

from requre.cassette import Cassette
from requre.constants import DEFAULT_IMPORT_FUNCTION
from requre.record_and_replace import (
    ModuleRecord,
    _parse_and_replace_sys_modules,
    _revert_modules,
)
from requre.storage import PersistentObjectStorage


class ReplaceType(Enum):
    """
    Types for import system what are used in replacement list, to know ho to replace it:
    DECORATOR: decorate original function
    REPLACE: replace object by another one
    """

    DECORATOR = 1
    REPLACE = 2


def decorate(
    what: str,
    decorator: Callable,
) -> "UpgradeImportSystem":
    """
    Decorate the function when importing.

    :param what: what will be decorated
    :param decorator: decorator that will be used
    :return: UpgradeImportSystem
    """
    upgraded_import_system = UpgradeImportSystem()
    upgraded_import_system.decorate(what=what, decorator=decorator)
    return upgraded_import_system


def replace(
    what: str,
    replacement: Any,
) -> "UpgradeImportSystem":
    """
    Replace the module when importing.

    :param what: what will be replaced
    :param replacement: what will be used instead
    :return: UpgradeImportSystem
    """
    upgraded_import_system = UpgradeImportSystem()
    upgraded_import_system.replace(what=what, replacement=replacement)
    return upgraded_import_system


class UpgradeImportSystem:
    def __init__(self, cassette: Optional[Cassette] = None) -> None:
        self.cassette = cassette or PersistentObjectStorage().cassette
        self.module_list: List[ModuleRecord] = []

    def __enter__(self) -> "UpgradeImportSystem":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        builtins.__import__ = DEFAULT_IMPORT_FUNCTION
        self.revert()

    def revert(self) -> "UpgradeImportSystem":
        """
        Revert the changes to the import system.

        :return: self (chaining is supported)
        """
        builtins.__import__ = DEFAULT_IMPORT_FUNCTION
        _revert_modules(self.module_list)
        # TODO: fix reverting of 'from' modules
        self.module_list.clear()
        return self

    def decorate(self, what: str, decorator: Callable) -> "UpgradeImportSystem":
        """
        Decorate the function when importing.

        :param what: what will be decorated
        :param decorator: decorator that will be used
        :return: self (chaining is supported)
        """
        return self.upgrade(what, ReplaceType.DECORATOR, decorator)

    def replace(self, what: str, replacement: Any) -> "UpgradeImportSystem":
        """
        Replace the module when importing.

        :param what: what will be replaced
        :param replacement: what will be used instead
        :return: self (chaining is supported)
        """

        return self.upgrade(what, ReplaceType.REPLACE, replacement)

    def upgrade(self, what, replace_type, replacement, add_revert_list=[]):
        """
        Apply the upgrade of modules (already loaded or when import of the module comes)

        :param what: what to update
        :param replace_type: type of replacement ReplaceType
        :param replacement: replaced or decorator to be applied
        :param add_revert_list: append anoher part where revert has to happen.
                                In case of some from statements it may lead that
                                they are not properly reverted as well.
        :return: self (chaining is supported)
        """
        self._upgrade_filter(what, replace_type, replacement, add_revert_list)
        return self

    def _upgrade_filter(
        self,
        what,
        replace_type,
        replacement,
        add_revert_list,
        func=DEFAULT_IMPORT_FUNCTION,
    ):
        replace = None
        decorate = None
        if replace_type == ReplaceType.REPLACE:
            replace = replacement
        elif replace_type == ReplaceType.DECORATOR:
            decorate = replacement
        # check if already loaded inside sys.modules, then do not apply update of import system
        recorded_items = _parse_and_replace_sys_modules(
            what=what,
            cassette=self.cassette,
            decorate=decorate,
            replace=replace,
            add_revert_list=add_revert_list,
        )
        if recorded_items:
            self.module_list += recorded_items
            return

        @functools.wraps(func)
        def new_import(*args, **kwargs):
            with open("debug.file", "w") as fd:
                fd.write("eee", what, args, kwargs)
            out = func(*args, **kwargs)
            recorded_items = _parse_and_replace_sys_modules(
                what=what,
                cassette=self.cassette,
                decorate=decorate,
                replace=replace,
                add_revert_list=add_revert_list,
            )
            if recorded_items:
                self.module_list += recorded_items
            return out

        func = new_import



# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

from requre.simple_object import Simple as _Simple
from requre.simple_object import Void as _Void
from requre.simple_object import Tuple as _Tuple

# BACKWARDS COMPATIBILITY
Simple = _Simple
Void = _Void
Tuple = _Tuple



# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

"""
Exceptions for requre project
"""


class PersistentStorageException(Exception):
    """Exceptions for persistent storage of objects"""


class ItemNotInStorage(PersistentStorageException):
    pass


class StorageNoResponseLeft(PersistentStorageException):
    pass



# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

import functools
import logging
import os
import tarfile
from io import BytesIO
from typing import Any, Dict, Optional, Type, Union

from requre.cassette import Cassette, CassetteExecution, StorageMode
from requre.exceptions import PersistentStorageException
from requre.guess_object import Guess
from requre.objects import ObjectStorage
from requre.simple_object import Simple

logger = logging.getLogger(__name__)


def return_cls_type(cassette: Optional[Cassette]) -> Union[Type[Simple], Type[Guess]]:
    if cassette and cassette.storage_file_version < 3:
        return Simple
    else:
        return Guess


class StoreFiles(ObjectStorage):
    dir_suffix = "file_storage"
    tar_compression = "xz"
    basic_ps_keys = ["X", "file", "tar"]
    _cassette: Cassette = None

    @staticmethod
    def _test_identifier(cassette):
        return os.path.basename(cassette.storage_file)

    @staticmethod
    def store_file_content(cassette: Cassette, content: Any, file_name: str):
        generated = f"{os.path.basename(cassette.storage_file)}.{file_name}_"
        target_dir = os.path.realpath(os.path.dirname(cassette.storage_file))
        # generate unique number for each file store to avoid rewrite current content
        # increase highest number
        suffix = f".tar.{StoreFiles.tar_compression}"
        existing_nums = sorted(
            int(x.replace(generated, "").replace(suffix, ""))
            for x in os.listdir(target_dir)
            if x.startswith(generated)
        )
        next_num = 1 if not existing_nums else existing_nums[-1] + 1
        file_path = os.path.join(target_dir, f"{generated}{next_num}{suffix}")
        with open(file_path, "wb") as outfile:
            outfile.write(content)
        # return properly generated unique file name
        return os.path.basename(file_path)

    @staticmethod
    def read_file_content(
        cassette: Cassette, file_name: str, root_dir: Optional[str] = None
    ):
        file_path = os.path.realpath(
            os.path.join(os.path.dirname(cassette.storage_file), file_name)
        )
        if not os.path.exists(file_path):
            raise FileNotFoundError(
                f"requre cannot read from file {file_path}, does not existsin dir: {root_dir})"
            )
        with open(file_path, "rb") as infile:
            content = infile.read()
        return content

    @classmethod
    def __write_file(cls, content, pathname):
        with BytesIO(content) as fileobj:
            with tarfile.open(
                mode=f"r:{cls.tar_compression}", fileobj=fileobj
            ) as tar_store:
                members = tar_store.getmembers()
                tarinfo_1st_member = members[0]
                if tarinfo_1st_member.isfile():
                    with open(pathname, mode="wb") as output_file:
                        output_file.write(
                            tar_store.extractfile(tarinfo_1st_member).read()
                        )
                    return
                for tar_item in members:
                    # we have to modify path of files to remove topdir
                    if len(tar_item.name.split(os.path.sep, 1)) > 1:
                        tar_item.name = tar_item.name.split(os.path.sep, 1)[1]
                    else:
                        tar_item.name = "."
                    try:
                        tar_store.extract(tar_item, path=pathname)
                    except OSError:
                        # rewrite readonly files if necessary
                        os.remove(os.path.join(pathname, tar_item.name))
                        tar_store.extract(tar_item, path=pathname)

    @classmethod
    def _copy_logic(
        cls,
        cassette: Cassette,
        pathname: str,
        keys: list,
        ret_store_cls: Any,
        return_value: Any,
    ) -> Any:
        """
        Internal function. Copy files to or back from persisten storage
        It will create tar archive with tar_compression and stores it to Persistent Storage
        """
        FILENAME = "filename"
        TARGET_PATH = "target_path"
        RETURNED = "return_value"
        serialization = ret_store_cls(store_keys=["not_important"], cassette=cassette)
        logger.debug(f"Copy files {pathname} -> {keys}")
        logger.debug(f"Persistent Storage mode: {cassette.mode}")
        original_cwd = os.getcwd()
        if cassette.do_store(keys=cls.basic_ps_keys + keys):
            try:
                artifact_name = os.path.basename(pathname)
                artifact_path = os.path.dirname(pathname)
                os.chdir(artifact_path)
                with BytesIO() as fileobj:
                    with tarfile.open(
                        mode=f"x:{cls.tar_compression}", fileobj=fileobj
                    ) as tar_store:
                        tar_store.add(name=artifact_name)
                    metadata = {cassette.data_miner.LATENCY_KEY: 0}
                    file_name = cls.store_file_content(
                        cassette=cassette,
                        file_name=os.path.basename(pathname),
                        content=fileobj.getvalue(),
                    )
                    serialized = serialization.to_serializable(return_value)
                    output = {
                        FILENAME: file_name,
                        RETURNED: serialized,
                        TARGET_PATH: pathname,
                    }
                    cassette.store(
                        keys=cls.basic_ps_keys + keys,
                        values=output,
                        metadata=metadata,
                    )
            finally:
                os.chdir(original_cwd)
        else:
            output = cassette[cls.basic_ps_keys + keys]
            pathname = pathname or output[RETURNED]
            content = cls.read_file_content(
                cassette=cassette, file_name=output[FILENAME]
            )
            # WORKAROUND: some parts expects old dir and some new one. so copy to both to ensure.
            # mainly when creating complex objects.
            for item in [pathname, output[TARGET_PATH]]:
                cls.__write_file(content, item)
            return_value = serialization.from_serializable(output[RETURNED])
        return return_value

    @classmethod
    def __get_executor_and_return_cls(cls, cassette, output_cls):
        casex = CassetteExecution()
        casex.cassette = cassette or cls.get_cassette()
        casex.obj_cls = cls
        if not output_cls:
            output_cls = return_cls_type(cassette)
        return casex, output_cls

    @classmethod
    def where_file_as_return_value(
        cls,
        cassette: Optional[Cassette] = None,
        output_cls: Optional[Any] = None,
    ) -> Any:
        """
        Decorator what will store return value of function/method as file and will store content

        :param cassette: Cassette instance to pass inside object to work with
        :return: CassetteExecution class with function and cassette instance

        """
        casex, output_cls = cls.__get_executor_and_return_cls(cassette, output_cls)

        def internal(func):
            @functools.wraps(func)
            def store_files_int(*args, **kwargs):
                output = None
                if casex.cassette.mode != StorageMode.read:
                    output = func(*args, **kwargs)
                output = cls._copy_logic(
                    cassette=casex.cassette,
                    pathname=output,
                    keys=[cls.__name__, cls._test_identifier(casex.cassette)],
                    ret_store_cls=output_cls,
                    return_value=output,
                )
                return output

            return store_files_int

        casex.function = internal
        return casex

    @classmethod
    def guess_files_from_parameters(
        cls,
        cassette: Optional[Cassette] = None,
        output_cls: Optional[Any] = None,
    ) -> Any:
        """
        Decorator what try to guess, which arg is file or directory and store its content

        :param cassette: Cassette instance to pass inside object to work with
        :return: CassetteExecution class with function and cassette instance
        """
        casex, output_cls = cls.__get_executor_and_return_cls(cassette, output_cls)

        def internal(func):
            @functools.wraps(func)
            def store_files_int(*args, **kwargs):
                output = None
                if casex.cassette.mode != StorageMode.read:
                    output = func(*args, **kwargs)

                def int_dec_fn(pathname_arg, keys_arg):
                    if not isinstance(pathname_arg, str):
                        return
                    if casex.cassette.do_store(
                        keys=StoreFiles.basic_ps_keys + keys_arg
                    ):
                        if os.path.exists(pathname_arg):
                            return cls._copy_logic(
                                cassette=casex.cassette,
                                pathname=pathname_arg,
                                keys=keys_arg,
                                ret_store_cls=output_cls,
                                return_value=output,
                            )
                    else:
                        try:
                            return cls._copy_logic(
                                cassette=casex.cassette,
                                pathname=pathname_arg,
                                keys=keys_arg,
                                ret_store_cls=output_cls,
                                return_value=output,
                            )
                        except PersistentStorageException:
                            pass

                class_test_id_list = [
                    cls.__name__,
                    cls._test_identifier(casex.cassette),
                ]
                for position in range(len(args)):
                    output = int_dec_fn(args[position], class_test_id_list + [position])
                for k, v in kwargs.items():
                    output = int_dec_fn(v, class_test_id_list + [k])
                return output

            return store_files_int

        casex.function = internal
        return casex

    @classmethod
    def where_arg_references(
        cls,
        key_position_params_dict: Dict,
        cassette: Optional[Cassette] = None,
        output_cls: Optional[Any] = None,
    ) -> Any:
        """
        Decorator what will store files or directory based on arguments,
        you have to pass name and position of arg via dict
        (be careful about counting self or cls parameteres for methods)
        eg. key_position_params_dict = {"target_dir": 2}

        :param cassette: Cassette instance to pass inside object to work with
        :return: CassetteExecution class with function and cassette instance
        """
        casex, output_cls = cls.__get_executor_and_return_cls(cassette, output_cls)

        def internal(func):
            @functools.wraps(func)
            def store_files_int_int(*args, **kwargs):
                output = None
                class_test_id_list = [
                    cls.__name__,
                    cls._test_identifier(casex.cassette),
                ]
                if casex.cassette.mode != StorageMode.read:
                    output = func(*args, **kwargs)
                for key, position in key_position_params_dict.items():
                    if key in kwargs:
                        param = kwargs[key]
                    else:
                        param = args[position]
                    output = cls._copy_logic(
                        cassette=casex.cassette,
                        pathname=param,
                        keys=class_test_id_list + [key],
                        ret_store_cls=output_cls,
                        return_value=output,
                    )
                return output

            return store_files_int_int

        casex.function = internal
        return casex

    @classmethod
    def explicit_reference(
        cls,
        file_param: str,
        dest_key: str = "default",
        cassette: Optional[Cassette] = None,
        output_cls: Optional[Any] = None,
    ) -> Any:
        """
        Method to store explicitly path file_param to persistent storage

        :param cassette: Cassette instance to pass inside object to work with
        :return: CassetteExecution class with function and cassette instance
        """
        casex, output_cls = cls.__get_executor_and_return_cls(cassette, output_cls)

        def internal(func):
            @functools.wraps(func)
            def store_files_int(*args, **kwargs):
                output = None
                class_test_id_list = [
                    cls.__name__,
                    cls._test_identifier(casex.cassette),
                ]
                if casex.cassette.mode != StorageMode.read:
                    output = func(*args, **kwargs)
                output = cls._copy_logic(
                    cassette=casex.cassette,
                    pathname=file_param,
                    keys=class_test_id_list + [dest_key],
                    ret_store_cls=output_cls,
                    return_value=output,
                )
                return output

            return store_files_int

        casex.function = internal
        return casex



# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

from requre.helpers.git.helper import record_git_module as _record_git_module
from requre.helpers.tempfile import record_tempfile_module as _record_tempfile_module
from requre.online_replacing import (
    record_requests_for_all_methods,
)

# BACKWARDS COMPATIBILITY
record_requests_module = record_requests_for_all_methods
record_tempfile_module = _record_tempfile_module
record_git_module = _record_git_module



# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

from requre.cassette import Cassette
from .singleton import SingletonMeta


class PersistentObjectStorage(metaclass=SingletonMeta):
    def __init__(self):
        super().__init__()
        self.cassette = Cassette()

    def __getattr__(self, item):
        return getattr(self.cassette, item)



# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

from requre.import_system import UpgradeImportSystem
from requre.helpers.tempfile import TempFile

FILTERS = UpgradeImportSystem().decorate(
    what="tempfile.mktemp", decorator=TempFile.mktemp()
)



# Copyright Contributors to the Packit project.
# SPDX-License-Identifier: MIT

import builtins
from requre.import_system import UpgradeImportSystem, replace, decorate
from tests.testbase import BaseClass
from tempfile import mktemp as original_mktemp


class TestUpgradeImportSystem(BaseClass):
    def setUp(self) -> None:
        super().setUp()
        self.old_imports = builtins.__import__

    def tearDown(self) -> None:
        builtins.__import__ = self.old_imports
        super().tearDown()

    def test_no_filter(self):
        old_imports = builtins.__import__
        with UpgradeImportSystem():
            import tempfile

            self.assertNotIn("decorated", tempfile.mktemp())
            self.assertIn("/tmp", tempfile.mktemp())
            tempfile.mktemp = original_mktemp
        assert builtins.__import__ == old_imports

    def test_method_replace(self):
        """
        Test improving of import system with import statement
        Check also debug file output if it contains proper debug data
        """

        UpgradeImportSystem().replace(what="tempfile.mktemp", replacement=lambda: "a")
        import tempfile

        self.assertNotIn("/tmp", tempfile.mktemp())
        self.assertIn("a", tempfile.mktemp())
        tempfile.mktemp = original_mktemp

    def test_method_context_replace(self):
        """
        Test improving of import system with import statement
        Check also debug file output if it contains proper debug data
        """
        with UpgradeImportSystem().replace(
            what="tempfile.mktemp", replacement=lambda: "b"
        ):
            import tempfile

            self.assertNotIn("/tmp", tempfile.mktemp())
            self.assertIn("b", tempfile.mktemp())
            tempfile.mktemp = original_mktemp

    def test_function_context_replace(self):
        """
        Test improving of import system with import statement
        Check also debug file output if it contains proper debug data
        """
        with replace(what="tempfile.mktemp", replacement=lambda: "c"):
            import tempfile

            self.assertNotIn("/tmp", tempfile.mktemp())
            self.assertIn("c", tempfile.mktemp())
            tempfile.mktemp = original_mktemp

    def test_method_decorate(self):
        """
        Test patching by decorator_all_keys, not replacing whole function
        """
        UpgradeImportSystem().decorate(
            what="tempfile.mktemp", decorator=lambda x: lambda: f"decorated_a {x()}"
        )
        import tempfile

        self.assertIn("decorated_a", tempfile.mktemp())
        self.assertIn("/tmp", tempfile.mktemp())
        tempfile.mktemp = original_mktemp

    def test_method_context_decorate(self):
        """
        Test patching by decorator_all_keys, not replacing whole function
        """
        with UpgradeImportSystem().decorate(
            what="tempfile.mktemp", decorator=lambda x: lambda: f"decorated_b {x()}"
        ):
            import tempfile

            self.assertIn("decorated_b", tempfile.mktemp())
            self.assertIn("/tmp", tempfile.mktemp())
            tempfile.mktemp = original_mktemp

    def test_function_context_decorate(self):
        """
        Test patching by decorator_all_keys, not replacing whole function
        """
        with decorate(
            what="tempfile.mktemp", decorator=lambda x: lambda: f"decorated_c {x()}"
        ):
            import tempfile

            self.assertIn("decorated_c", tempfile.mktemp())
            self.assertIn("/tmp", tempfile.mktemp())
            tempfile.mktemp = original_mktemp
